﻿using System;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.Mvc;
using FileApiBlockBlobWebRole.Models;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;

namespace FileApiBlockBlobWebRole.Controllers
{
    public class HomeController : Controller
    {
        public ActionResult Index()
        {
            ViewBag.Message = "";
            ViewBag.Error = false;
            return View();
        }

        [HttpPost]
        public ActionResult Index(HttpPostedFileBase file)
        {
            try
            {
                if (file != null && file.ContentLength > 0)
                {
                    var container = CloudStorageAccount
                        .Parse(CloudConfigurationManager.GetSetting("StorageAccountConnectionString"))
                        .CreateCloudBlobClient()
                        .GetContainerReference("uploads");
                    container.CreateIfNotExist();

                    var fileName = Path.GetFileName(file.FileName);

                    var blob = container.GetBlockBlobReference(fileName);
                    blob.Properties.ContentType = GetMimeType(Path.GetExtension(file.FileName));
                    blob.UploadFromStream(file.InputStream, new BlobRequestOptions { RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromSeconds(10)) });
                }
                ViewBag.Message = "File upload completed successfully.";
                ViewBag.Error = false;
            }
            catch (Exception ex)
            {
                ViewBag.Message = ex.Message;
                ViewBag.Error = true;
            }

            return View("Index");
        }

        [HttpPost]
        public ActionResult PrepareUpload(int blocksCount, string fileName, long fileSize, string fileType)
        {
            try
            {
                var container = CloudStorageAccount
                    .Parse(CloudConfigurationManager.GetSetting("StorageAccountConnectionString"))
                    .CreateCloudBlobClient()
                    .GetContainerReference("uploads");
                container.CreateIfNotExist();

                var fileToUpload = new FileUploadModel
                {
                    BlockCount = blocksCount,
                    FileName = fileName,
                    FileSize = fileSize,
                    FileType = fileType,
                    BlockBlob = container.GetBlockBlobReference(fileName)
                };

                Session.Add("FileAttributesSession", fileToUpload);

                return Json(new { error = false, message = string.Empty });
            }
            catch (Exception ex)
            {
                return Json(new { error = true, message = ex.Message });
            }
        }

        [HttpPost]
        [ValidateInput(false)]
        public ActionResult UploadBlock(int id)
        {
            var chunk = new byte[Request.InputStream.Length];
            Request.InputStream.Read(chunk, 0, Convert.ToInt32(Request.InputStream.Length));
            if (Session["FileAttributesSession"] != null)
            {
                var model = (FileUploadModel)Session["FileAttributesSession"];
                using (var chunkStream = new MemoryStream(chunk))
                {
                    var blockId = Convert.ToBase64String(Encoding.UTF8.GetBytes(string.Format(CultureInfo.InvariantCulture, "{0:D4}", id)));
                    try
                    {
                        model.BlockBlob.PutBlock(blockId, chunkStream, null, new BlobRequestOptions { RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromSeconds(10)) });
                    }
                    catch (StorageException e)
                    {
                        return Json(new { error = true, isLastBlock = false, message = e.Message });
                    }
                }

                if (id == model.BlockCount)
                {
                    try
                    {
                        var blockList = Enumerable.Range(1, model.BlockCount).ToList().ConvertAll(rangeElement => Convert.ToBase64String(Encoding.UTF8.GetBytes(string.Format(CultureInfo.InvariantCulture, "{0:D4}", rangeElement))));
                        model.BlockBlob.Properties.ContentType = model.FileType;
                        model.BlockBlob.PutBlockList(blockList);
                    }
                    catch (StorageException e)
                    {
                        return Json(new { error = true, isLastBlock = true, message = e.Message });
                    }
                    finally
                    {
                        Session.Clear();
                    }

                    return Json(new { error = false, isLastBlock = true, message = "File upload completed successfully." });
                }
            }
            else
            {
                return Json(new { error = true, isLastBlock = false, message = "Failed To Upload File. Session expired." });
            }
            return Json(new { error = false, isLastBlock = false, message = string.Empty });
        }

        private static string GetMimeType(string extension)
        {
            var mimeType = String.Empty;

            var regKey = Microsoft.Win32.Registry.ClassesRoot.OpenSubKey(extension);
            if (regKey != null)
            {
                var type = (string)regKey.GetValue("Content Type");
                if (type != null) mimeType = type;
            }
            return mimeType;
        }
    }
}
