﻿var jqxhr;

function resetControls(){
    $('#uploadProgress').hide();
    $('#uploadButton').show();
    $('#cancelButton').hide();
}

function renderProgress(currentValue, maxValue) {
    $('#uploadProgress')
        .attr({ value: currentValue, max: maxValue })
        .show();
}

function startUpload() {
    var files = document.getElementById('fileInput').files;
    if (!files) {
        alert("Your browser doesn't support the HTML 5 File API!");
        return;
    }
    if (!files.length) {
        alert('Please select a file!');
        return;
    }

    var blockLength = $('#blockLengthSelector').val();

    $('#statusMessage')
        .removeClass('alert alert-error alert-block alert-success alert-info')
        .text('');
    $('#uploadButton').hide();
    $('#cancelButton').show();

    var totalBlocks = Math.ceil(files[0].size / blockLength);

    var sendFile = function(blockSize) {
        var start = 0,
            end = Math.min(blockSize, files[0].size),
            blockId = 1,
            retryCount = 0,
            maxRetries = 3,
            retryAfterSeconds = 10;

        var sendNextBlock = function() {
            var fileBlock = new window.FormData();
            renderProgress(blockId, totalBlocks);
            if (files[0].slice) {
                fileBlock.append('Slice', files[0].slice(start, end));
            } else if (files[0].webkitSlice) {
                fileBlock.append('Slice', files[0].webkitSlice(start, end));
            } else if (files[0].mozSlice) {
                fileBlock.append('Slice', files[0].mozSlice(start, end));
            } else {
                $('#statusMessage')
                    .removeClass('alert alert-error alert-block alert-success alert-info')
                    .addClass('alert alert-error')
                    .text("This Browser is not supported.");
                return;
            }
            jqxhr = $.ajax({
                async: true,
                type: 'POST',
                url: ('/Home/UploadBlock/' + blockId),
                data: fileBlock,
                cache: false,
                contentType: false,
                processData: false,
                error: function(request, error) {
                    if (error !== 'abort' && retryCount < maxRetries) {
                        retryCount++;
                        setTimeout(sendNextBlock, retryAfterSeconds * 1000);
                    }
                    if (error === 'abort') {
                        $('#statusMessage')
                            .removeClass('alert alert-error alert-block alert-success alert-info')
                            .addClass('alert alert-info')
                            .text("The upload has been aborted.");
                        resetControls();
                    } else {
                        if (retryCount === maxRetries) {
                            $('#statusMessage')
                                .removeClass('alert alert-error alert-block alert-success alert-info')
                                .addClass('alert-error')
                                .text("Failed to upload file. Max retries exceeded.");
                            resetControls();
                        } else {
                            $('#statusMessage')
                                .removeClass('alert alert-error alert-block alert-success alert-info')
                                .addClass('alert alert-block')
                                .text("Failed to upload block. (" + retryCount + " of " + maxRetries + " retries)");
                        }
                    }
                    return;
                },
                success: function(notice) {
                    if (notice.error || notice.isLastBlock) {
                        if (notice.error)
                            $('#statusMessage')
                                .removeClass('alert alert-error alert-block alert-success alert-info')
                                .addClass('alert alert-error')
                                .text(notice.message);
                        else
                            $('#statusMessage')
                                .removeClass('alert alert-error alert-block alert-success alert-info')
                                .addClass('alert alert-success')
                                .text(notice.message);
                        resetControls();
                        return;
                    }

                    blockId++;
                    start = (blockId - 1) * blockSize;
                    end = Math.min(blockId * blockSize, files[0].size) - 1;
                    retryCount = 0;
                    sendNextBlock();
                }
            });
        };

        $('#statusMessage')
            .removeClass('alert alert-error alert-block alert-success alert-info')
            .addClass('alert alert-info')
            .text("Uploading file...");
        sendNextBlock();
    };

    $('#statusMessage')
        .removeClass('alert alert-error alert-block alert-success alert-info')
        .addClass('alert alert-info')
        .text("Preparing file upload...");
    $.ajax({
        type: "POST",
        async: true,
        url: '/Home/PrepareUpload',
        data: {
            'blocksCount': totalBlocks,
            'fileName': files[0].name,
            'fileSize': files[0].size,
            'fileType': files[0].type
        },
        dataType: "json",
        error: function() {
            $('#statusMessage')
                .removeClass('alert alert-error alert-block alert-success alert-info')
                .addClass('alert alert-error')
                .text("Unable to prepare the upload.");
            resetControls();
        },
        success: function(notice) {
            if (notice.error) {
                $('#statusMessage')
                    .removeClass('alert alert-error alert-block alert-success alert-info')
                    .addClass('alert alert-error')
                    .text(notice.message);
                resetControls();
                return;
            } else {
                sendFile(blockLength);
            }
        }
    });
}

function cancelUpload() {
    if (jqxhr !== null) {
        jqxhr.abort();
    }
}

$().ready(function () {
    $('#uploadProgress').hide();
    $('#cancelButton').hide();
    if (window.File && window.FileReader && window.FileList && window.Blob) {
        $('#uploadButton').click(startUpload);
        $('#cancelButton').click(cancelUpload);
        $('#submitButton').hide();
        $('#blockLengthGroup').show();
    } else {
        $('#uploadButton').hide();
        $('#blockLengthGroup').hide();
    }
});